package polyglot.ext.j0.types;

import polyglot.types.*;
import polyglot.ext.jl.types.*;

import java.util.*;

public class J0TypeSystem_c extends TypeSystem_c implements J0TypeSystem {
    // TODO: implement new methods in SkelTypeSystem.
    // TODO: override methods as needed from TypeSystem_c.

    public final HashMap nameMap; // String -> HashMap((List of Type) -> MethodInstance)

    
    public J0TypeSystem_c() {
	super();

	nameMap = new HashMap();
    }

    public Context createContext() {
	return new J0Context_c(this);
    }

    public MethodInstance findStaticMethod(String name, List argTypes) throws SemanticException {
	HashMap methodMap = (HashMap)nameMap.get(name);

	if(methodMap == null) {
	    throw new SemanticException("Method " + name + " not found.");
	}

	Collection keys = methodMap.keySet();

	Iterator keyIterator = keys.iterator();

	boolean different = false;

	while(keyIterator.hasNext()) {
	    List argTs = (List)keyIterator.next();

	    ListIterator lIterator = argTs.listIterator();
	    ListIterator rIterator = argTypes.listIterator();

	    while(lIterator.hasNext() && rIterator.hasNext()) {
		Type l = (Type)lIterator.next();
		Type r = (Type)rIterator.next();

		if(!equals(l, r)) {
		    different = true;
		    break;
		}
	    }

	    if(!different && !lIterator.hasNext() && !rIterator.hasNext()) {
		return (MethodInstance)methodMap.get(argTs);
	    }

	}
	
	throw new SemanticException("Method " + name + " not found.");
    }

    public void addStaticMethod(String qualifiedName, String name, List argTypes, MethodInstance mi) throws SemanticException {

	Flags flags = mi.flags();

	if(!flags.isStatic()) {
	    throw new SemanticException("Can only import a static method. " + name + " is not static.");
	}

	HashMap methodMap = (HashMap)nameMap.get(name);
	HashMap instanceMap; 
	if(methodMap == null) {
	    methodMap = new HashMap();
	    nameMap.put(name, methodMap);
	    
	} else {

	    Collection keys = methodMap.keySet();
	
	    Iterator keyIterator = keys.iterator();

	    boolean different = false;

	    while(keyIterator.hasNext()) {
		List argTs = (List)keyIterator.next();

		ListIterator lIterator = argTs.listIterator();
		ListIterator rIterator = argTypes.listIterator();

		while(lIterator.hasNext() && rIterator.hasNext()) {
		    Type l = (Type)lIterator.next();
		    Type r = (Type)rIterator.next();

		    if(!equals(l, r)) {
			different = true;
			break;
		    }
		}

		if(!different && !lIterator.hasNext() && !rIterator.hasNext()) {
		    throw new SemanticException("Cannot import two static methods with the same name and parameter types: " + name);
		}

	    }
	}

	mi = mi.name(qualifiedName);

	methodMap.put(argTypes, mi);

    }
    
    

}
